@@ -38,6 +38,9 @@ from elisa.plugins.base.local_resource import LocalResource
 from elisa.plugins.base.models.file import DirectoryModel
 
 
+# FIXME: we put here also schema stuff not strictly related with the media
+# scanner (the "favorites" table), because we don't have a general
+# mechanism to do database stuff yet
 SCHEMA = [
         "CREATE TABLE IF NOT EXISTS music_albums" \
                 " (name VARCHAR PRIMARY KEY, cover_uri VARCHAR," \
@@ -62,10 +65,21 @@ SCHEMA = [
                 " (file_path VARCHAR, tag_name VARCHAR,"\
                 " PRIMARY KEY(file_path, tag_name));",
         "CREATE TABLE IF NOT EXISTS images"\
-                " (file_path VARCHAR PRIMARY KEY, size VARCHAR);",
+                " (file_path VARCHAR PRIMARY KEY, size VARCHAR, " \
+                " shot_time INTEGER, with_flash INTEGER DEFAULT 0," \
+                " orientation INTEGER, gps_altitude INTEGER, " \
+                " gps_latitude VARCHAR, gps_longitude VARCHAR," \
+                " album_name VARCHAR);",
+        "CREATE TABLE IF NOT EXISTS photo_albums"\
+                " (name VARCHAR PRIMARY KEY, preview_uri VARCHAR);",
         "CREATE TABLE IF NOT EXISTS videos"\
                 " (file_path VARCHAR PRIMARY KEY, size VARCHAR,"\
-                " duration VARCHAR, codec VARCHAR, thumbnail_uri VARCHAR);"
+                " duration VARCHAR, codec VARCHAR, thumbnail_uri VARCHAR);",
+
+        "CREATE TABLE IF NOT EXISTS favorites"\
+                " (id INTEGER PRIMARY KEY, section TEXT,"\
+                " added_time TEXT, title TEXT, subtitle,"\
+                " foreign_id TEXT, foreign_class TEXT, uri TEXT);",
         ]
 
 class MediaScanner(ResourceProvider):
@@ -100,12 +114,13 @@ class MediaScanner(ResourceProvider):
         if not directories_section:
             self.debug("no default directories")
             return
-        paths = directories_section.get('music', [])
-        for path in paths:
-            if path != '*default*': # skip the default configuration thingy
-                uri = MediaUri('file://%s' % path)
-                self.debug("adding %s" % uri)
-                self.put(uri, None)
+        for place in ('music', 'pictures'):
+            paths = directories_section.get(place, [])
+            for path in paths:
+                if path != '*default*': # skip the default configuration thingy
+                    uri = MediaUri('file://%s' % path)
+                    self.debug("adding %s" % uri)
+                    self.put(uri, None)
 
     def initialize(self):
 
@@ -169,6 +184,20 @@ class MediaScanner(ResourceProvider):
             if commit:
                 return self.store.commit()
 
+        def got_images(result):
+            if not 'shot_time' in result:
+                # shot_time is part of the first update we did for the image
+                # table. as it is not there, we have to update the table
+                # FIXME: this way to identify an Database Update is not nice
+                sqlexec("ALTER TABLE images ADD shot_time INTEGER;")
+                sqlexec("ALTER TABLE images ADD with_flash INTEGER DEFAULT 0;")
+                sqlexec("ALTER TABLE images ADD orientation INTEGER;")
+                sqlexec("ALTER TABLE images ADD gps_altitude INTEGER;")
+                sqlexec("ALTER TABLE images ADD gps_latitude VARCHAR;")
+                sqlexec("ALTER TABLE images ADD gps_longitude VARCHAR;")
+                sqlexec("ALTER TABLE images ADD album_name VARCHAR;")
+                return self.store.commit()
+
         def got_albums(result):
             if not u'release_date' in result:
                 sqlexec("ALTER TABLE music_albums ADD release_date VARCHAR;")
@@ -184,8 +213,14 @@ class MediaScanner(ResourceProvider):
             dfr.addCallback(got_albums)
             return dfr
 
+        def do_images_update(old_res=None):
+            dfr = get_table_info("images")
+            dfr.addCallback(got_images)
+            return dfr
+
         dfr = do_files_update()
         dfr.addCallback(do_albums_update)
+        dfr.addCallback(do_images_update)
         return dfr
 
     def create_schema(self):
