@@ -18,7 +18,7 @@
 
 from elisa.plugins.database.database_parser import DatabaseParser
 from elisa.plugins.database.models import File, MusicTrack, MusicAlbum, \
-        Artist, Tag, Image, Video
+        Artist, Tag, Image, PhotoAlbum, Video
 
 from elisa.core.media_uri import MediaUri
 
@@ -99,8 +99,7 @@ class MetadataDummy(object):
 
 class Dummy(object):pass
 
-class TestDatabaseParser(DBMixin, TestCase):
-
+class ParserSetupMixin(DBMixin):
     def setUp(self):    
         def parser_done(parser):
             self.parser = parser
@@ -113,10 +112,13 @@ class TestDatabaseParser(DBMixin, TestCase):
             self.parser.metadata = self.metadata
 
             return metadata.clean()
-        dfr = super(TestDatabaseParser, self).setUp()
+        dfr = super(ParserSetupMixin, self).setUp()
         dfr.addCallback(lambda x: DatabaseParser.create({}))
         dfr.addCallback(parser_done)
         return dfr
+ 
+
+class TestGeneralFileParsing(ParserSetupMixin, TestCase):
 
     def test_modification_time_the_same(self):
 
@@ -176,6 +178,108 @@ class TestDatabaseParser(DBMixin, TestCase):
         self.parser.update_modification_time(Failure(NotImplementedError()), file)
         self.failIf(file.modification_time == -1, "Modifcation time not changed")
 
+    def test_set_mime_type(self):
+
+        def check(res, file):
+            self.assertEquals(file.mime_type, u'test')
+
+        def parse_it(track, file):
+            tags = {'mime_type':u'test'}
+
+            return self.parser.parse_tags(tags, file).addCallback(check, file)
+
+        file = File()
+        file.path = u"testcase"
+        return self.store.add(file).addCallback(parse_it, file)
+
+    def test_deleted(self):
+
+        class DummySource(object):pass
+
+        def mark_as_deleted(result, source):
+            return self.parser.mark_deleted(source)
+
+        def check_deleted(result, file):
+            self.assertTrue(file.deleted, "Was not deleted")
+
+        file = File()
+        file.path = u"testcase"
+        file.source = u'/tmp'
+
+        return self.store.add(file).addCallback(mark_as_deleted, u'/tmp'
+                ).addCallback(check_deleted, file)
+
+
+class TestFindingRightTypeParsing(ParserSetupMixin, TestCase):
+    """
+    check if the parser at least starts with the right entries in the database
+    for the given file_type (the least gstreamer can give us)
+    """
+
+    def test_image(self):
+        """
+        check whether file_type:image leads to having an image entry
+        """
+
+        def check(res):
+
+            def got_image(image):
+                self.failUnless(image, "Image not in DB")
+            
+            return self.store.get(Image, u'test').addCallback(got_image)
+
+        def parse_it(res, file):
+            tags = {'file_type' : 'image'}
+            return self.parser.parse_tags(tags, file)
+
+        file = File()
+        file.path = u"test"
+        dfr = self.store.add(file)
+        return dfr.addCallback(parse_it, file).addCallback(check)
+
+    def test_video(self):
+        """
+        check whether file_type:video leads to having an video entry
+        """
+
+        def check(res):
+
+            def got_video(video):
+                self.failUnless(video, "Video not in DB")
+            
+            return self.store.get(Video, u'test').addCallback(got_video)
+
+        def parse_it(res, file):
+            tags = {'file_type' : 'video'}
+            return self.parser.parse_tags(tags, file)
+
+        file = File()
+        file.path = u"test"
+        dfr = self.store.add(file)
+        return dfr.addCallback(parse_it, file).addCallback(check)
+
+    def test_audio(self):
+        """
+        test whether file_type:audio leads to having the musictrack entry
+        """
+
+        def check(res):
+
+            def got_track(track):
+                self.failUnless(track, "MusicTrack not in DB")
+            
+            return self.store.get(MusicTrack, u'test').addCallback(got_track)
+
+        def parse_it(res, file):
+            tags = {'file_type' : 'audio'}
+            return self.parser.parse_tags(tags, file)
+
+        file = File()
+        file.path = u"test"
+        dfr = self.store.add(file)
+        return dfr.addCallback(parse_it, file).addCallback(check)
+
+class TestMusicParsing(ParserSetupMixin, TestCase):
 
     def test_with_new_album_and_new_artist(self):
 
@@ -373,20 +477,6 @@ class TestDatabaseParser(DBMixin, TestCase):
         return self.store.add(file).addCallback(add_track).addCallback(
                 parse_it, file)
 
-    def test_set_mime_type(self):
-
-        def check(res, file):
-            self.assertEquals(file.mime_type, u'test')
-
-        def parse_it(track, file):
-            tags = {'mime_type':u'test'}
-
-            return self.parser.parse_tags(tags, file).addCallback(check, file)
-
-        file = File()
-        file.path = u"testcase"
-        return self.store.add(file).addCallback(parse_it, file)
-
     def test_empty(self):
 
         def check(res):
@@ -448,6 +538,298 @@ class TestDatabaseParser(DBMixin, TestCase):
         file.path = u"test"
         dfr = self.store.add(file)
         return dfr.addCallback(parse_it, file).addCallback(check)
+
+class TestImageParsing(ParserSetupMixin, TestCase):
+
+    def test_with_new_album(self):
+
+        def check(res):
+
+            def got_album(album):
+                self.assertEquals(album.name, 'winter 2006')
+
+            def got_image(image):
+                date = datetime.datetime(2008, 6, 11, 12, 4, 59)
+                self.assertEquals(image.shot_time, date)
+                # not stored in the db
+                #self.assertEquals(image.device_make, "Fluendo")
+                #self.assertEquals(image.device_model, "Elisa Capture Cam")
+                self.assertEquals(image.with_flash, True)
+                self.assertEquals(image.orientation, 1)
+                self.assertEquals(image.gps_altitude, 12)
+                self.assertEquals(image.gps_latitude, '41,24,0N')
+                self.assertEquals(image.gps_longitude, '2,10,0E')
+                return image.album.addCallback(got_album)
+            
+            return self.store.get(Image, u'something/winter 2006/test'
+                    ).addCallback(got_image)
+
+        def parse_it(res, file):
+            tags = {'date-time-original' : '2008:06:11 12:04:59',
+                    'device-make' : 'Fluendo',
+                    'device-model' : 'Elisa Capture Cam',
+                    'capture-flash' : 1,
+                    'capture-orientation' : 1,
+                    'gps-altitude' : 12,
+                    'gps-latitude' : '41,24,0N',
+                    'gps-longitude' :  '2,10,0E',
+                    'file_type' : 'image',
+                    }
+            return self.parser.parse_tags(tags, file)
+
+        file = File()
+        file.path = u"something/winter 2006/test"
+        dfr = self.store.add(file)
+        return dfr.addCallback(parse_it, file).addCallback(check)
+
+    def test_dates(self):
+
+        def test_one(key, value, expected):
+            def got_image(image, expected):
+                self.assertEquals(image.shot_time, expected)
+            
+            def check(res, key, expected):
+                return self.store.get(Image, u'test%s' % key).addCallback(got_image,
+                        expected)
+
+            def parse_it(res, file):
+                tags = {'file_type': 'image', key : value}
+                return self.parser.parse_tags(tags, file)
+
+            file = File()
+            file.path = u"test%s" % key
+            dfr = self.store.add(file)
+            return dfr.addCallback(parse_it, file).addCallback(check, key, expected)
+
+        listing = [ ('date-time-modified', '2008-06-11 12:04:59',
+                            datetime.datetime(2008, 6, 11, 12, 4, 59)),
+                    ('date-time-original', '2007-05-17T18:09:12',
+                            datetime.datetime(2007, 5, 17, 18, 9, 12)),
+                    ('date-time-digitized', '2006-04-19T19:25:59',
+                            datetime.datetime(2006, 4, 19, 19, 25, 59)),
+                    ]
+
+        def iterate(to_test):
+            for key, value, result in to_test:
+                yield test_one(key, value, result)
+
+        dfr = task.coiterate(iterate(listing))
+        return dfr
+
+
+    def test_uses_the_best_date(self):
+        # if it can choose original is the best one
+        tags = {'file_type' : 'image',
+                'date-time-modified':'2008:06:11 12:04:59',
+                'date-time-original': '2007:05:17 18:09:12',
+                'date-time-digitized': '2006:04:19 19:25:59',}
+
+        def got_image(image):
+            self.assertEquals(image.shot_time,
+                    datetime.datetime(2007, 5, 17, 18, 9, 12))
+        
+        def check(res):
+            return self.store.get(Image, u'test').addCallback(got_image)
+
+        def parse_it(res, file, tags):
+            return self.parser.parse_tags(tags, file)
+
+        file = File()
+        file.path = u"test"
+        dfr = self.store.add(file)
+        return dfr.addCallback(parse_it, file, tags).addCallback(check)
+
+
+    def test_reuse_album(self):
+
+        def add_album(res):
+            album = PhotoAlbum()
+            album.name = u'test_album'
+            return self.store.add(album)
+
+        def check(result, ref_album):
+
+            def got_album(album, ref):
+                self.failUnless(album is ref, "Album not reused")
+            
+            def got_image(image, ref_album):
+                return image.album.addCallback(got_album,
+                        ref_album)
+
+            return self.store.get(Image, u'test_album/testcase'
+                    ).addCallback(got_image, ref_album)
+
+        def parse_it(ref, file):
+            tags = {'file_type' : 'image'}
+            return self.parser.parse_tags(tags, file).addCallback(check, ref)
+
+        file = File()
+        file.path = u"test_album/testcase"
+        return self.store.add(file).addCallback(add_album).addCallback(
+                parse_it, file)
+
+    def test_reuse_image(self):
+
+        def add_image(file):
+            image = Image()
+            image.file_path = file.path
+            return self.store.add(image)
+
+        def check(res, reference):
+            
+            def got_image(image, reference):
+                self.failUnless(image is reference, "Track was not reused")
+
+            return self.store.get(Image, u'testcase'
+                        ).addCallback(got_image, reference)
+
+        def parse_it(image, file):
+            tags = { 'file_type' : 'image'}
+
+            return self.parser.parse_tags(tags, file).addCallback(check, image)
+
+        file = File()
+        file.path = u"testcase"
+        return self.store.add(file).addCallback(add_image).addCallback(
+                parse_it, file)
+
+    def test_empty(self):
+
+        def check(res):
+
+            def got_album(album):
+                self.assertEquals(album.name, u'alabum')
+
+            def got_image(image):
+                self.assertEquals(image.shot_time, None)
+                # not stored in the database
+                #self.assertEquals(image.device_make, None)
+                #self.assertEquals(image.device_model, None)
+                self.assertEquals(image.with_flash, None)
+                self.assertEquals(image.orientation, None)
+                self.assertEquals(image.gps_altitude, None)
+                self.assertEquals(image.gps_latitude, None)
+                self.assertEquals(image.gps_longitude, None)
+                return image.album.addCallback(got_album)
+            
+            return self.store.get(Image, u'alabum/test').addCallback(got_image)
+
+        def parse_it(res, file):
+            tags = {'file_type' : 'image'}
+            return self.parser.parse_tags(tags, file)
+
+        file = File()
+        file.path = u"alabum/test"
+        dfr = self.store.add(file)
+        return dfr.addCallback(parse_it, file).addCallback(check)
+
+    def test_album_parsing(self):
+
+        def test_one(path, albumname): 
+
+            def check(res):
+
+                def got_album(album):
+                    self.assertEquals(album.name, albumname)
+
+                def got_image(image):
+                   return image.album.addCallback(got_album)
+                
+                return self.store.get(Image, path).addCallback(got_image)
+
+            def parse_it(res, file):
+                tags = {'file_type' : 'image',
+                        'date-time-original' : None,
+                        'device-make' : None,
+                        'device-model' : None,
+                        'capture-flash' : None,
+                        'capture-orientation' : None,
+                        'gps-altitude' : None,
+                        'gps-latitude' : None,
+                        'gps-longitude' : None,
+                        }
+                return self.parser.parse_tags(tags, file)
+
+            file = File()
+            file.path = path
+            dfr = self.store.add(file)
+            return dfr.addCallback(parse_it, file).addCallback(check)
+
+        listing = [ (u'/home/ben/Bilder/test.jpg', u'Bilder'),
+                    (u'C://Users/Test/nice.jpg', u'Test')]
+
+        def iterate(paths):
+            for path, albumname in paths:
+                yield test_one(path, albumname)
+
+        dfr = task.coiterate(iterate(listing))
+        return dfr
+
+    def test_no_album(self):
+    
+        def check(res):
+            def got_album(album):
+                self.failIf(album, "There should not be any album...")
+
+            def got_image(image):
+               return image.album.addCallback(got_album)
+            
+            return self.store.get(Image, u"test").addCallback(got_image)
+
+        def parse_it(res, file):
+            tags = {'file_type' : 'image'}
+            return self.parser.parse_tags(tags, file)
+
+        file = File()
+        file.path = u'test'
+        dfr = self.store.add(file)
+        return dfr.addCallback(parse_it, file).addCallback(check)
+
+
+    def test_none_values(self):
+
+        def check(res):
+
+            def got_album(album):
+                self.assertEquals(album.name, u'albumli')
+
+            def got_image(image):
+                self.assertEquals(image.shot_time, None)
+                #self.assertEquals(image.device_make, None) # not stored
+                #self.assertEquals(image.device_model, None)
+                self.assertEquals(image.with_flash, None)
+                self.assertEquals(image.orientation, None)
+                self.assertEquals(image.gps_altitude, None)
+                self.assertEquals(image.gps_latitude, None)
+                self.assertEquals(image.gps_longitude, None)
+                return image.album.addCallback(got_album)
+            
+            return self.store.get(Image, u'path/to/albumli/test').addCallback(got_image)
+
+        def parse_it(res, file):
+            tags = {'file_type' : 'image',
+                    'date-time-original' : None,
+                    'device-make' : None,
+                    'device-model' : None,
+                    'capture-flash' : None,
+                    'capture-orientation' : None,
+                    'gps-altitude' : None,
+                    'gps-latitude' : None,
+                    'gps-longitude' : None,
+                    }
+            return self.parser.parse_tags(tags, file)
+
+        file = File()
+        file.path = u"path/to/albumli/test"
+        dfr = self.store.add(file)
+        return dfr.addCallback(parse_it, file).addCallback(check)
+
+
+
+class TestTagging(ParserSetupMixin, TestCase):
+
+    todo = "The tagging system is deactivated"
+
     def test_audio_tagged(self):
 
         def check(res, file):
@@ -523,57 +905,4 @@ class TestDatabaseParser(DBMixin, TestCase):
         return self.store.add(file).addCallback(add_tag, file
                 ).addCallback(parse_it, file).addCallback(check, file)
 
-    def test_image(self):
-
-        def check(res):
-
-            def got_image(image):
-                self.failUnless(image, "Image not in DB")
-            
-            return self.store.get(Image, u'test').addCallback(got_image)
-
-        def parse_it(res, file):
-            tags = {'file_type' : 'image'}
-            return self.parser.parse_tags(tags, file)
-
-        file = File()
-        file.path = u"test"
-        dfr = self.store.add(file)
-        return dfr.addCallback(parse_it, file).addCallback(check)
-
-    def test_video(self):
-
-        def check(res):
-
-            def got_video(video):
-                self.failUnless(video, "Video not in DB")
-            
-            return self.store.get(Video, u'test').addCallback(got_video)
-
-        def parse_it(res, file):
-            tags = {'file_type' : 'video'}
-            return self.parser.parse_tags(tags, file)
-
-        file = File()
-        file.path = u"test"
-        dfr = self.store.add(file)
-        return dfr.addCallback(parse_it, file).addCallback(check)
-
-    def test_deleted(self):
-
-        class DummySource(object):pass
-
-        def mark_as_deleted(result, source):
-            return self.parser.mark_deleted(source)
-
-        def check_deleted(result, file):
-            self.assertTrue(file.deleted, "Was not deleted")
-
-        file = File()
-        file.path = u"testcase"
-        file.source = u'/tmp'
-
-        return self.store.add(file).addCallback(mark_as_deleted, u'/tmp'
-                ).addCallback(check_deleted, file)
-
 
