@@ -21,6 +21,7 @@ gobject.threads_init()
 import gst
 import os
 import time
+import platform
 
 from twisted.internet import defer, reactor
 
@@ -32,7 +33,7 @@ from elisa.plugins.gstreamer.thumbnail import PngImageSnapshotBin, \
 from elisa.plugins.gstreamer import thumbnail
  
 supported_metadata_keys = set(['artist', 'album', 'song', 'track', \
-        'thumbnail','date'])
+        'thumbnail','date', 'date-time-modified'])
 media_type_keys = set(['uri', 'file_type', 'mime_type'])
 thumbnail_keys = set(['uri', 'thumbnail'])
 supported_keys = supported_metadata_keys.union(media_type_keys)
@@ -46,6 +47,47 @@ class TimeoutError(MetadataError):
 class GstMetadataError(MetadataError):
     pass
 
+def tweak_gstreamer_elements():
+    # load the plugins so we can override the ranks... yuck!
+    for plugin_name in ('typefindfunctions', 'png', 'jpeg', 'ffmpeg', 'soup'):
+        gst.plugin_load_by_name(plugin_name)
+
+    registry = gst.registry_get_default()
+    typefinders = gst.type_find_factory_get_list()
+    for typefinder in typefinders:
+        name = typefinder.get_name()
+
+        if name in ('image/jpeg', 'image/png'):
+            # override the rank of the jpeg and png typefinders in order to speed up
+            # typefinding when browsing Pictures
+            typefinder.set_rank(gst.RANK_PRIMARY + 1)
+        elif name in ('fftype_wsaud',):
+            # we don't really want to typefind .py files as audio files...
+            registry.remove_feature(typefinder)
+        elif name == 'audio/x-m4a':
+            # okay, this typefinder is a bit annoying. If it finds the
+            # ftypmp42 atom it suggests with _MAXIMUM probability that it's
+            # an audio/x-m4a. This means that all youtube high definition
+            # videos are found as audio. 
+            # We simply remove the typefinder here, as the file will still
+            # be recognized by the video/quicktime typefinder and demuxed by
+            # qtdemux anyway.
+            registry.remove_feature(typefinder)
+        elif platform.system() == 'Windows' and \
+                    name in  ('application/x-ape', 'application/x-apetag'):
+                registry.remove_feature(typefinder)
+    
+    # raise the rank of souphttpsrc as it's better than gnomevfs for http
+    soupfactory = gst.element_factory_find('souphttpsrc')
+    if soupfactory:
+        soupfactory.set_rank(gst.RANK_PRIMARY + 1)
+
+    # raise the rank of metadatademux to retrieve exif tags
+    metadata_factory = gst.element_factory_find('metadatademux')
+    if metadata_factory:
+        metadata_factory.set_rank(gst.RANK_PRIMARY + 1)
+
+    
 
 class GstMetadataPipeline(Loggable):
     reuse_elements = False
@@ -79,6 +121,7 @@ class GstMetadataPipeline(Loggable):
             self._seek_call = None
 
     def initialize(self):
+        tweak_gstreamer_elements()
         self._reset()
 
     def _clean_pipeline(self, finalize=False):
@@ -253,10 +296,16 @@ class GstMetadataPipeline(Loggable):
             file_type = None
 
         req_keys = set(self._req_metadata.keys())
-        if (req_keys == media_type_keys and \
-                file_type in ('video', 'audio', 'image'))or \
-                (file_type in ('video', 'image') and \
-                (not 'thumbnail' in req_keys or self._have_thumbnail())):
+
+        if req_keys == media_type_keys and file_type in \
+                    ('video', 'audio', 'image') or \
+                file_type == 'video' and \
+                    (not 'thumbnail' in req_keys or  \
+                    self._have_thumbnail()) or \
+                (file_type == 'image' and \
+                    req_keys.intersection(supported_keys) == thumbnail_keys \
+                    and  self._have_thumbnail()):
+
             self.debug('got media_type for %s, NOT going to paused',
                     self._req_uri)
             # we are in the streaming thread so we post a message on the bus
@@ -483,7 +532,7 @@ class GstMetadataPipeline(Loggable):
                 self._video_caps = caps
                 # do a thumbnail of the first video track
                 self._plug_video_thumbnailbin(pad)
-        elif typ == 'image':
+        elif typ == 'image' and "thumbnail" in self._req_metadata:
             if self._image_caps is None:
                 self._image_caps = caps
                 self._plug_image_thumbnailbin(pad)
@@ -595,9 +644,13 @@ class GstMetadataPipeline(Loggable):
         if 'date' in available_metadata:
             # we need to rewrite the date to a timestamp
             date = available_metadata['date']
-            time_stamp = time.mktime( (date.year, date.month, date.year, 0, \
-                                        0, 0, 0, 0, -1) )
-            available_metadata['date'] = time_stamp
+            if date is not None:
+                try:
+                    time_stamp = time.mktime((date.year, date.month, \
+                                              date.day, 0, 0, 0, 0, 0, -1))
+                    available_metadata['date'] = time_stamp
+                except (OverflowError, ValueError):
+                    pass
 
         for key, value in available_metadata.iteritems():
             try:
