@@ -20,9 +20,10 @@ setup(name='elisa-plugin-ipod',
     entry_points="""\
     """,
     controller_mappings=[('/poblesec/music/ipod',
-                         'elisa.plugins.ipod.controller:ArtistsListSwitcherVertical'),
+                          'elisa.plugins.ipod.controller:ArtistsIpodListSwitcherController'),
                          ('/poblesec/music/ipod/albums',
-                         'elisa.plugins.ipod.controller:AlbumsListSwitcherCoverflow'),
+                          'elisa.plugins.ipod.controller:AlbumsIpodListSwitcherController'),
                          ('/poblesec/music/ipod/tracks',
-                         'elisa.plugins.ipod.controller:TracksListSwitcherVertical'),],
+                          'elisa.plugins.ipod.controller:TracksIpodListSwitcherController'),],
+    decorator_mappings=[],
     cmdclass=cmdclass)
