@@ -29,7 +29,8 @@ from elisa.core.utils.dist import read_mappings
 from twisted.internet import reactor, task, defer
 import os, platform
 import pgm
-from pgm.timing import controller, ticker
+from pgm.timing.controller import Controller
+from pgm.timing.ticker import Ticker
 import pkg_resources
 from elisa.plugins.pigment.pigment_input import PigmentInput
 import weakref
@@ -377,9 +378,9 @@ class PigmentFrontend(Frontend, PigmentFrontendExtensionMixin):
         # best solution to update Pigment animations because it's synchronized
         # with the rendering. Note that this is the only Pigment signal which
         # is emitted in another thread than the mainloop one.
-        tck = ticker.Ticker()
-        controller.Controller.set_ticker(tck)
-        id = viewport.connect('update-pass', self._update_pass_callback, tck)
+        ticker = Ticker()
+        Controller.set_ticker(ticker)
+        id = viewport.connect('update-pass', self._update_pass_callback, ticker)
         self._signal_handler_ids.append(id)
 
         # hide the cursor
