@@ -17,12 +17,14 @@
 from elisa.plugins.pigment.pigment_controller import PigmentController
 from elisa.plugins.pigment.graph.image import Image
 from elisa.plugins.pigment.graph.text import Text
+from elisa.plugins.pigment.graph.group import Group
 from elisa.plugins.pigment.widgets.widget import Widget
 from elisa.plugins.pigment.widgets.progressbar import ProgressBar
 from elisa.plugins.pigment.widgets.const import *
 from elisa.core.input_event import *
 from elisa.core.media_uri import MediaUri
 from elisa.core.utils.i18n import install_translation
+import platform
 
 try:
     from elisa.plugins.database.models import File as DBFile
@@ -46,6 +48,75 @@ import gst
 from gst import pbutils
 import math
 import os
+import platform
+
+class VisualisationError(Exception):
+    pass
+
+# The Visualisation Fake
+class ElisaVisualisationBin(gst.Bin):
+
+    def __init__(self, visualisation, size, pgm_image):
+        """
+        The ElisaVisualisationBin is a wrapper for the different visualisation
+        elements of GStreamer.
+
+        @param visualisation:   the GStreamer Element to use
+        @type visualisation:    string
+        @param size:            a size in pixel to calculate the visualisation
+                                size for
+        @type size:             int
+        @param pgm_image:       the pgm_image that is used for sinking
+        @type pgm_image:         L{pgm.graph.image.Image}
+        """
+
+        gst.Bin.__init__(self)
+
+        self._visualisation = visualisation
+        self._pgm_image = pgm_image
+
+        self._capsfilter = gst.element_factory_make('capsfilter')
+        self.add(self._capsfilter)
+
+        caps_pad = self._capsfilter.get_pad('src')
+
+        self._visu = gst.element_factory_make(self._visualisation)
+        self.add(self._visu)
+
+        height = int(round(size * pgm_image.absolute_height / pgm_image.absolute_width))
+        height = (height + 3) & ~3
+
+        visu_sink = self._visu.get_pad('sink')
+        visu_src = self._visu.get_pad('src')
+
+        caps = visu_src.get_pad_template().caps
+
+        if caps == None:
+            raise VisualisationError("The Visualisation element %s does not" \
+                                     " have any Caps." % self._visualisation)
+
+        caps = caps.copy()
+
+        if caps.is_fixed() == False:
+            caps.make_writable()
+            for i in xrange(0, caps.get_size()):
+                cur_struct = caps.get_structure(i)
+
+                cur_struct.fixate_field_nearest_int('width', size)
+
+                cur_struct.fixate_field_nearest_int('height', height)
+        else:
+            raise VisualisationError("The Caps of the GstElement are fixed.")
+
+        self._capsfilter.set_property('caps', caps)
+
+        self._ghost_sink = gst.GhostPad('sink', visu_sink )
+        self._ghost_src = gst.GhostPad('src', caps_pad)
+
+        self._visu.link(self._capsfilter)
+        self.add_pad(self._ghost_sink)
+        self.add_pad(self._ghost_src)
+
 
 class PlayerOsdExit(Widget):
 
@@ -72,12 +143,6 @@ class PlayerOsdStatus(Widget):
     def __init__(self):
         super(PlayerOsdStatus, self).__init__()
 
-        self.dock = Image()
-        self.add(self.dock)
-        self.dock.bg_a = 0
-        self.dock.alignment = pgm.IMAGE_TOP_LEFT
-        self.dock.visible = True
-
         self.play_button = Image()
         self.add(self.play_button)
         self.play_button.bg_a = 0
@@ -89,27 +154,51 @@ class PlayerOsdStatus(Widget):
         self.pause_button.bg_a = 0
         self.pause_button.alignment = pgm.IMAGE_LEFT
         self.pause_button.visible = False
+        
+        self.seeking_dock_group = Group()
+        self.add(self.seeking_dock_group)
+        self.seeking_dock_group.visible = True
+        
+        self.dock = Image()
+        self.seeking_dock_group.add(self.dock)
+        self.dock.connect("pixbuf-loaded", self._body_image_loaded)
+        self.dock.bg_a = 0
+        self.dock.alignment = pgm.IMAGE_TOP_LEFT
+        self.dock.visible = True
 
         self.text = Text()
-        self.add(self.text)
+        self.seeking_dock_group.add(self.text)
         self.text.bg_a = 0
         self.text.ellipsize = pgm.TEXT_ELLIPSIZE_END
         self.text.visible = True
 
         self.counter = Text()
-        self.add(self.counter)
+        self.seeking_dock_group.add(self.counter)
         self.counter.bg_a = 0
         self.counter.alignment = pgm.TEXT_ALIGN_RIGHT
         self.counter.visible = True
 
         self.progress_bar = ProgressBar()
-        self.add(self.progress_bar)
+        self.seeking_dock_group.add(self.progress_bar)
         self.progress_bar.bg_a = 0
         self.progress_bar.visible = True
         self.progress_bar._orientation = HORIZONTAL
 
         self._update_style_properties(self._style.get_items())
         self.play()
+        
+    def _body_image_loaded(self, img):
+        self.dock_image_loaded = True
+        drawable_aspect_ratio = self.dock.absolute_width / \
+                                            float(self.dock.absolute_height)
+        image_aspect_ratio = self.dock.aspect_ratio[0] / \
+                                            float(self.dock.aspect_ratio[1])
+
+        #FIXME : does not work when Elisa has an aspect ratio < 1 need theme
+        #update for this case.
+        #previous_size = self.seeking_dock_group.size
+        self.seeking_dock_group.width = image_aspect_ratio / \
+                                                float(drawable_aspect_ratio)
 
     def _update_style_properties(self, props=None):
         super(PlayerOsdStatus, self)._update_style_properties(props)
@@ -379,6 +468,17 @@ class Player(gobject.GObject, Loggable):
     def __init__(self):
         super(Player, self).__init__()
         # GStreamer pipeline setup
+
+        # make sure typefindfunctions is loaded so we can remove the typefinders
+        if platform.system() == 'Windows':
+            gst.plugin_load_by_name('typefindfunctions')
+            registry = gst.registry_get_default()
+            typefinders = gst.type_find_factory_get_list()
+            for typefinder in typefinders:
+                if typefinder.get_name() in ('application/x-ape',
+                    'application/x-apetag'):
+                    registry.remove_feature(typefinder)
+
         self.pgm_sink = gst.element_factory_make('pgmimagesink')
         self.pipeline = gst.element_factory_make('playbin')
         self.pipeline.set_property('video-sink', self.pgm_sink)
@@ -465,6 +565,7 @@ class Player(gobject.GObject, Loggable):
                     break
         if not found:
             self.info("No subtitles found for %r", uri)
+            self.pipeline.set_property('suburi', '')
 
 
     def set_drawable(self, value):
@@ -527,11 +628,20 @@ class Player(gobject.GObject, Loggable):
             self.stop()
             self.pipeline.set_property('uri', model.uri)
             self._load_subs(model.uri)
+
+            # FIXME: make this configurable
+            vis_plugin_name = 'libvisual_jess'
+            if platform.system() == 'Windows':
+                vis_plugin_name = 'goom2k1'
+
             try:
-                visu = gst.element_factory_make('libvisual_jess')
+                visu = ElisaVisualisationBin(vis_plugin_name, 400,
+                                             self.image)
                 self.pipeline.set_property('vis-plugin', visu)
             except:
                 pass
+
+
             self.play()
             self._update_playcount_and_time(model)
             self.current_index = index
@@ -548,7 +658,8 @@ class Player(gobject.GObject, Loggable):
         current = self.pipeline.get_state()[1]
         if current != gst.STATE_PLAYING:
             if current != gst.STATE_PAUSED:
-                self.emit('status-changed', self.BUFFERING)
+                self.status = self.BUFFERING
+                self.emit('status-changed', self.status)
             self.pipeline.set_state(gst.STATE_PLAYING)
 
     def get_position(self):
@@ -878,18 +989,23 @@ class PlayerController(PigmentController):
 
     def _load_img_from_theme(self):
         lft = self.frontend.load_from_theme
-
+        theme = self.frontend.get_theme()
+        
         osd_status = self.player_osd.status
-        lft('elisa.plugins.poblesec.osd_status_dock',
-                                      osd_status.dock)
+        
+        img_path = theme.get_resource('elisa.plugins.poblesec.osd_status_dock')
+        osd_status.dock.set_from_file(img_path)
+        
         lft('elisa.plugins.poblesec.osd_status_play_button',
                                       osd_status.play_button)
         lft('elisa.plugins.poblesec.osd_status_pause_button',
                                       osd_status.pause_button)
 
         osd_volume = self.player_osd.volume
-        lft('elisa.plugins.poblesec.osd_volume_dock',
-                                      osd_volume.dock)
+        
+        img_path = theme.get_resource('elisa.plugins.poblesec.osd_volume_dock')
+        osd_volume.dock.set_from_file(img_path)
+        
         lft('elisa.plugins.poblesec.osd_volume_button',
                                       osd_volume.play_button)
 
