@@ -26,6 +26,8 @@ from elisa.core.input_event import *
 
 from pgm.timing import implicit
 import pgm
+import gobject
+
 
 class TopBarButton(Widget):
     
@@ -288,6 +290,10 @@ class Crumb(Widget):
 
 class BreadCrumbs(Widget):
 
+    __gsignals__ = {'crumb-clicked': (gobject.SIGNAL_RUN_LAST,
+                                      gobject.TYPE_BOOLEAN,
+                                      (gobject.TYPE_PYOBJECT,))}
+
     def __init__(self):
         super(BreadCrumbs, self).__init__()
         self._crumbs = []
@@ -345,6 +351,7 @@ class BreadCrumbs(Widget):
             text = _('unknown')
         crumb = Crumb()
         crumb.text.label = text
+        crumb.background.connect('clicked', lambda *args: self.emit('crumb-clicked', crumb))
         self._crumbs.append(crumb)
         self.add(crumb)
         crumb.background.set_from_image(self.master)
@@ -474,6 +481,8 @@ class PoblesecBrowserController(BrowserController):
         #create the top bar with bread crumb
         self.topbar_with_bread_crumb = TopBarWithBreadCrumbs()
         self.widget.add(self.topbar_with_bread_crumb)
+        self.topbar_with_bread_crumb.bread_crumb.connect('crumb-clicked',
+                                                         self._bread_crumb_clicked)
         self.topbar_with_bread_crumb.visible = True
 
         #hide it
@@ -504,6 +513,13 @@ class PoblesecBrowserController(BrowserController):
 
         bb.connect('focus', self._button_bar_focus_changed)
 
+    def _bread_crumb_clicked(self, bread_crumbs, crumb):
+        if not self.history:
+            return
+
+        while bread_crumbs._crumbs[-1] != crumb:
+            self.history.go_back()
+
     def _button_bar_focus_changed(self, button_bar, focus):
         if focus:
             button_bar.set_index(0)
@@ -525,12 +541,19 @@ class PoblesecBrowserController(BrowserController):
                             + self.topbar_with_bread_crumb.bread_crumb.height 
             controller.widget.x, controller.widget.y = (0.0, y)
             controller.widget.width, controller.widget.height = (1.0, 1.0 - y)
+
+    def _go_section_start(self, *args):
+        if self.history:
+            while self.history.index > 1:
+                self.history.go_back()
      
     def _push_controller_signal(self, history, previous, current):
         if self.history.index > 0:
             if self.history.index == 1:
                 topbar = self.topbar_with_bread_crumb.topbar
                 topbar.text.label = current.display_name
+                topbar.image.connect('clicked', self._go_section_start)
+                topbar.text.connect('clicked', self._go_section_start)
                 image_path = previous.model[previous.menu.selected_item_index].icon
                 self.frontend.load_from_theme(image_path, topbar.image)
                  
